@@ -50,7 +50,7 @@ ZBlobSystem  *z_blob_system_default = NULL;
 
 /* And its default attributes
  * (Don't worry about the names, they will be used only 3 times...)*/
-const gchar *z_blob_system_default_tmpdir = "/var/lib/zorp/tmp/"; /**< directory to store the blobs in */
+const gchar *z_blob_system_default_tmpdir = ZORPLIB_TEMP_DIR;     /**< directory to store the blobs in */
 gint64 z_blob_system_default_max_disk_usage = 1024*0x100000;      /**< max disk usage = 1 GB */
 gsize z_blob_system_default_max_mem_usage = 256*0x100000;         /**< max mem usage = 256 MB */
 gsize z_blob_system_default_lowat = 96*0x100000;                  /**< lowat = 96 MB */
