@@ -31,7 +31,9 @@
 #ifdef G_OS_WIN32
 #  include <winsock2.h>
 #include <io.h>
-#define close _close 
+/* NOTE: on windows handles opened with socket() must call closesocket()
+         and must NOT call any other close that triggers undefined behavior... */
+#define close closesocket
 #else
 #  include <netinet/tcp.h>
 #  include <netinet/in.h>
@@ -53,6 +55,7 @@ z_connector_connected(gboolean timed_out, gpointer data)
 {
   ZConnector *self = (ZConnector *) data;
   int error_num = 0;
+  const gchar * error_num_str = NULL;
   socklen_t errorlen = sizeof(error_num);
   ZConnectFunc callback;
   GError *err = NULL;
@@ -66,6 +69,7 @@ z_connector_connected(gboolean timed_out, gpointer data)
   if (timed_out)
     {
       error_num = ETIMEDOUT;
+      error_num_str = "connection timed out";
     }
   else if (getsockopt(fd, SOL_SOCKET, SO_ERROR, (char *)(&error_num), &errorlen) == -1)
     {
@@ -79,6 +83,9 @@ z_connector_connected(gboolean timed_out, gpointer data)
   if (error_num)
     {
       char buf1[MAX_SOCKADDR_STRING], buf2[MAX_SOCKADDR_STRING];
+      if (!error_num_str)
+        error_num_str = g_strerror(error_num);
+
       
       /*LOG
         This message indicates that the connection to the remote end
@@ -87,7 +94,7 @@ z_connector_connected(gboolean timed_out, gpointer data)
        */
       z_log(self->session_id, CORE_ERROR, 2, "Connection to remote end failed; local='%s', remote='%s', error='%s'", 
                self->local ? z_sockaddr_format(self->local, buf1, sizeof(buf1)) : "NULL", 
-               z_sockaddr_format(self->remote, buf2, sizeof(buf2)), g_strerror(error_num));
+               z_sockaddr_format(self->remote, buf2, sizeof(buf2)), error_num_str);
       
       /* self->poll.fd is closed when we are freed */
       fd = -1;
@@ -108,7 +115,7 @@ z_connector_connected(gboolean timed_out, gpointer data)
   if (self->watch)
     {
       if (error_num)
-        g_set_error(&err, 0, error_num, g_strerror(error_num));
+        g_set_error(&err, 0, error_num, "%s", error_num_str);
       
       callback = self->callback;
       self->callback = NULL;
@@ -563,14 +570,7 @@ ZConnectorFuncs z_connector_funcs =
 #ifdef G_OS_WIN32
   LIBZORPLL_EXTERN
 #endif 
-ZClass ZConnector__class = 
-{
-  Z_CLASS_HEADER,
-  Z_CLASS(ZObject),               // super_class 
-  "ZConnector",                   // name
-  sizeof(ZConnector),             // size
-  &z_connector_funcs.super        // funcs
-};
+Z_CLASS_DEF(ZConnector, ZObject, z_connector_funcs);
 
 /**
  * ZStreamConnector virtual methods.
