@@ -221,10 +221,10 @@ gboolean
 z_send_syslog(gint pri, const gchar *msg)
 {
   /*
-   * Log message max line length is 1024 (g_log max line length)
-   * + 128 for timestamp, process name and pid
+   * NOTE: We limit the log message size to 8192 now. Hope it will be enough.
+   * IIRC syslog-ng can only handle 8k long messages.
    */
-  gchar buf[2048];
+  gchar buf[8192];
   const guchar *p;
   gchar timestamp[32];
   time_t now;
@@ -880,18 +880,15 @@ z_logv(const gchar *class, int level, gchar *format, va_list ap)
   
   if (log_tags)
     {
-      /*
-       * g_log cut it in the 1024th byte,
-       * so it's enough to write this amount of data.
-       */
-      gchar msgbuf[1024];
-      g_vsnprintf(msgbuf, sizeof(msgbuf), format, ap);
+      gchar *msgbuf;
+      msgbuf = g_strdup_vprintf(format, ap);
 
 #if ZORPLIB_ENABLE_TRACE
       g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%p -> %s(%d): %s", g_thread_self(), class, level, msgbuf);
 #else
       g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s(%d): %s", class, level, msgbuf);
 #endif
+      g_free(msgbuf);
     }
   else
     {
