@@ -119,7 +119,7 @@ guint32 mem_trace_free_list = MEMTRACE_EOL;
 guint32 mem_block_count = 0, mem_allocated_size = 0, mem_alloc_count = 0;
 gboolean mem_trace_initialized = FALSE;
 static GStaticMutex mem_trace_lock = G_STATIC_MUTEX_INIT;
-gchar mem_trace_filename[1024] = "/var/tmp/zorp-memtrace.log";
+gchar mem_trace_filename[1024] = ZORPLIB_TEMP_DIR "/zorp-memtrace.log";
 gboolean mem_trace_canaries = FALSE;
 gboolean mem_trace_hard = FALSE;
 gboolean really_trace_malloc = FALSE;
@@ -168,13 +168,15 @@ z_mem_trace_init_internal(void)
   
   if (!mem_trace_initialized)
     {
+#if defined(__AMD64__) || defined(AMD64) || defined (__x86_64__)
       gpointer temp_buf[10];
+#endif /* __amd64__ */
 
       mem_trace_initialized = TRUE;
 
-  #if defined(__AMD64__) || defined(AMD64) || defined (__x86_64__)
+#if defined(__AMD64__) || defined(AMD64) || defined (__x86_64__)
       backtrace(temp_buf, 5);
-  #endif /* __amd64__ */
+#endif /* __amd64__ */
       
       z_mem_trace_init_pointers();
       
@@ -220,7 +222,7 @@ z_mem_trace_init(gchar *tracefile)
 
   if (tracefile && mem_trace)
     {
-      g_snprintf(mem_trace_filename, sizeof(mem_trace_filename), "/var/tmp/%s", tracefile);
+      g_snprintf(mem_trace_filename, sizeof(mem_trace_filename), ZORPLIB_TEMP_DIR "/%s", tracefile);
       z_mem_trace_printf("MemTrace initialized; memtrace='%s', canaries='%s', keep_deleted='%s', full_log='%s'\n",
                 YES_NO_STR(mem_trace), YES_NO_STR(mem_trace_canaries), YES_NO_STR(mem_trace_hard), YES_NO_STR(really_trace_malloc));
     }
@@ -704,7 +706,7 @@ z_malloc(size_t size, gpointer backt[])
       if (mem_trace_hard && z_mem_trace_getsize(user_ptr) != -1)
         {
           z_mem_trace_printf("Duplicate memory block; backtrace='%s'\n",
-                             z_mem_trace_format_bt(backtrace, buf, sizeof(buf)));
+                             z_mem_trace_format_bt(backt, buf, sizeof(buf)));
           abort();
         }
 
