@@ -159,6 +159,13 @@ z_pktbuf_insert(ZPktBuf *self, gsize pos, const void *data, gsize length)
   return TRUE;
 }
 
+gboolean
+z_pktbuf_data_equal(ZPktBuf *lhs, ZPktBuf *rhs)
+{
+  return lhs->length == rhs->length &&
+         !memcmp(lhs->data, rhs->data, rhs->length);
+}
+
 /**
  * Create a new ZPktBuf instance, set its length to 0 and its data ptr to NULL
  *
