@@ -32,16 +32,19 @@
 #include <assert.h>
 
 /**
- * ZPoll:
- *
+ * @file
  *
+ * @todo FIXME: this module should be discarded and GMainContext should be used
+ * directly instead. -- Bazsi, 2006.08.11. 
+ **/
+
+/**
  * ZRealPoll is an implementation of the ZPoll interface defined in
  * poll.h. It's a callback based poll loop, which can be used by proxy
  * modules. It holds a collection of ZStream objects and calls their
  * callbacks when a requested I/O event occurs. ZPoll objects are
  * reference counted and are automatically freed when the number of
  * references to a single instance reaches zero.
- *
  **/
 typedef struct _ZRealPoll
 {
@@ -52,13 +55,10 @@ typedef struct _ZRealPoll
   gboolean quit;
   GStaticMutex lock;
   GSource *wakeup;
-  GHashTable *streams;
 } ZRealPoll;
 
 /**
- * ZPollSource:
- *
- * This structure is a special source used to wake up a ZPoll loop from
+ * This class is a special source used to wake up a ZPoll loop from
  * different threads.
  **/
 typedef struct _ZPollSource
@@ -68,57 +68,63 @@ typedef struct _ZPollSource
 } ZPollSource;
 
 /**
- * z_poll_source_prepare:
- * @s: ZPollSource instance
- * @timeout: poll timeout
+ * This is the prepare function of ZPollSource.
+ *
+ * @param[in]  s ZPollSource instance
+ * @param[out] timeout poll timeout
  *
  * This is the prepare function of ZPollSource, it basically checks whether
  * the loop was woken up by checking self->wakeup, and returns TRUE in that
- * case. The timeout value returned is always -1, e.g. infinite timeout.
- **/    
+ * case.
+ *
+ * @see GSourceFuncs documentation.
+ *
+ * @returns always -1 timeout value, e.g. infinite timeout
+ **/
 static gboolean
 z_poll_source_prepare(GSource *s, gint *timeout)
 {
   ZPollSource *self = (ZPollSource *)s;
     
   z_enter();
-  
   if (self->wakeup)
-    {
-      return TRUE;
-    }
-  
+    z_return(TRUE);
+
   *timeout = -1;
-  z_leave();
-  return FALSE;
+  z_return(FALSE);
 }
 
 /**
- * z_poll_source_check:
- * @s: ZPollSource instance (not used)
+ * This is the check function of ZPollSource.
  *
- * This is the check function of ZPollSource. As we poll nothing we always
- * return FALSE.
+ * @param s ZPollSource instance (not used)
  *
- * FIXME: I think this function should check the value of self->wakeup
+ * As we poll nothing we always return FALSE.
+ *
+ * @see GSourceFuncs documentation.
+ *
+ * @returns FALSE
+ *
+ * @todo FIXME: I think this function should check the value of self->wakeup
  **/
 static gboolean
 z_poll_source_check(GSource *s G_GNUC_UNUSED)
 {
   z_enter();
-      
-  z_leave();
-  return FALSE;
+  z_return(FALSE);
 }
 
 /**
- * z_poll_source_dispatch:
- * @s: ZPollSource instance
- * @callback: the callback for @s (not used)
- * @user_data: the data to be passed to @callback (not used)
- *
  * This function simply sets self->wakeup to FALSE to allow the next poll loop
  * to run.
+ *
+ * @param[in] s ZPollSource instance
+ * @param     callback the callback for s (not used)
+ * @param     user_data the data to be passed to callback (not used)
+ *
+ * @see GSourceFuncs documentation.
+ *
+ * @returns TRUE
  **/
 static gboolean
 z_poll_source_dispatch(GSource *s,
@@ -128,13 +134,13 @@ z_poll_source_dispatch(GSource *s,
   ZPollSource *self = (ZPollSource *) s;
   
   z_enter();
-
   self->wakeup = FALSE;
-
-  z_leave();
-  return TRUE;
+  z_return(TRUE);
 }
 
+/**
+ * ZPollSource virtual methods.
+ **/
 GSourceFuncs z_poll_source_funcs = 
 {
   z_poll_source_prepare,
@@ -146,29 +152,9 @@ GSourceFuncs z_poll_source_funcs =
 };
 
 /**
- * z_poll_stream_unref:
- * @s: ZStream instance to free
- *
- * This function is used as the destroy notify callback of the hashtable in
- * ZPoll. It detaches the source of the stream by calling
- * z_stream_detach_source and also drops the reference to the stream.
- **/
-static void 
-z_poll_stream_unref(gpointer s)
-{
-  ZStream *stream = (ZStream *)s;
-  
-  z_enter();
-  z_stream_detach_source(stream);
-  z_stream_unref(stream);
-  z_leave();
-}
-
-/** z_poll_new:
- *
  * This function creates a new ZPoll instance.
  *
- * Returns: a pointer to the new instance
+ * @returns a pointer to the new instance
  **/
 ZPoll *
 z_poll_new(void)
@@ -177,16 +163,10 @@ z_poll_new(void)
   
   z_enter();
   g_return_val_if_fail( self != NULL, NULL);
-  
   self->ref_count = 1;
   self->quit = FALSE;
   self->pollfd_num = 4;
   self->pollfd = g_new(GPollFD, self->pollfd_num);
-  self->streams = g_hash_table_new_full(g_direct_hash,
-                                        g_direct_equal,
-                                        z_poll_stream_unref,
-                                        NULL);
-  
   self->context = g_main_context_default();
   if (g_main_context_acquire(self->context))
     {
@@ -197,27 +177,22 @@ z_poll_new(void)
       self->context = g_main_context_new();
       assert(g_main_context_acquire(self->context));
     }
-  
   self->wakeup = g_source_new(&z_poll_source_funcs,
                               sizeof(ZPollSource));
-  
   g_source_attach(self->wakeup, self->context);
-  
-  z_leave();
-  return (ZPoll *) self;
+  z_return((ZPoll *) self);
 }
 
 /** 
- * z_poll_destroy:
- * @s: ZPoll instance
- *
  * Used internally to free up an instance when the reference count
  * reaches 0.
+ *
+ * @param[in] s ZPoll instance
  **/
 static void
 z_poll_destroy(ZPoll *s)
 {
-  ZRealPoll *self = (ZRealPoll *)s;
+  ZRealPoll *self = (ZRealPoll *) s;
 
   z_enter();
   if(self->wakeup)
@@ -226,20 +201,17 @@ z_poll_destroy(ZPoll *s)
       g_source_unref(self->wakeup);
       self->wakeup = NULL;
     }
-  g_hash_table_destroy(self->streams);
   g_main_context_release(self->context);
   g_main_context_unref(self->context);
   g_free(self->pollfd);
-  
   g_free(self);
-  z_leave();
+  z_return();
 }
 
 /**
- * z_poll_ref:
- * @s: ZPoll instance
- *
  * Increment the reference count of the given ZPoll instance.
+ *
+ * @param[in] s ZPoll instance
  **/
 void 
 z_poll_ref(ZPoll *s)
@@ -248,16 +220,15 @@ z_poll_ref(ZPoll *s)
   
   z_enter();
   self->ref_count++;
-  z_leave();
+  z_return();
 }
 
 
 /**
- * z_poll_unref:
- * @s: ZPoll instance
- *
  * Decrement the reference count of the given ZPoll instance, and free it
  * using z_poll_destroy() if it reaches 0.
+ *
+ * @param[in] s ZPoll instance
  **/
 void
 z_poll_unref(ZPoll *s)
@@ -272,15 +243,14 @@ z_poll_unref(ZPoll *s)
       if (self->ref_count == 0)
         z_poll_destroy(s);
     }
-  z_leave();
+  z_return();
 }
 
 /**
- * z_poll_add_stream:
- * @s: ZPoll instance
- * @stream: stream instance
- *
  * Register a ZStream to be monitored by a ZPoll instance.
+ *
+ * @param[in] s ZPoll instance
+ * @param[in] stream stream instance
  **/
 void
 z_poll_add_stream(ZPoll *s, ZStream *stream)
@@ -288,42 +258,34 @@ z_poll_add_stream(ZPoll *s, ZStream *stream)
   ZRealPoll *self = (ZRealPoll *) s;
 
   z_enter();
-  
-  z_stream_ref(stream);
-  g_hash_table_insert(self->streams, stream, NULL);
   z_stream_attach_source(stream, self->context);
-  
-  z_leave();
+  z_return();
 }
 
 /**
- * z_poll_remove_stream:
- * @s: ZPoll instance
- * @stream: stream to be removed
- *
  * Remove a ZStream from a ZPoll instance.
+ *
+ * @param[in] s ZPoll instance
+ * @param[in] stream stream to be removed
  **/
 void
-z_poll_remove_stream(ZPoll *s, ZStream *stream)
+z_poll_remove_stream(ZPoll *s G_GNUC_UNUSED, ZStream *stream)
 {
-  ZRealPoll *self = (ZRealPoll *) s;
-  
   z_enter();
-  
-  g_hash_table_remove(self->streams, stream);
-
-  z_leave();
+  z_stream_detach_source(stream);
+  z_return();
 }
 
 /**
- * z_poll_iter_timeout:
- * @s: ZPoll instance
- * @timeout: timeout value in milliseconds
+ * Run an iteration of the poll loop.
  *
- * Run an iteration of the poll loop. Monitor filedescriptors of
+ * @param[in] s ZPoll instance
+ * @param[in] timeout timeout value in milliseconds
+ *
+ * This function runs an iteration of the poll loop. Monitor filedescriptors of
  * registered Streams, and call appropriate callbacks.
  *
- * Returns: TRUE if the iteration should be called again.
+ * @returns TRUE if the iteration should be called again.
  **/
 guint
 z_poll_iter_timeout(ZPoll *s, gint timeout)
@@ -338,13 +300,8 @@ z_poll_iter_timeout(ZPoll *s, gint timeout)
   z_enter();
   z_errno_set(0);
   if (self->quit)
-    {
-      z_leave();
-      return 0;
-    }
-  
+    z_return(0);
   g_main_context_prepare (self->context, &max_priority);
-
   fdnum = g_main_context_query(self->context,
                                max_priority,
                                &polltimeout,
@@ -359,7 +316,6 @@ z_poll_iter_timeout(ZPoll *s, gint timeout)
       z_log(NULL, CORE_DEBUG, 7, "Polling fd structure growing; old_num='%d'", self->pollfd_num);
       self->pollfd_num *= 2;
       self->pollfd = g_renew(GPollFD, self->pollfd, self->pollfd_num);
-      
       fdnum = g_main_context_query(self->context,
                                    max_priority,
                                    &polltimeout,
@@ -373,36 +329,28 @@ z_poll_iter_timeout(ZPoll *s, gint timeout)
     polltimeout = MIN(polltimeout, timeout);
 
   pollfunc = g_main_context_get_poll_func(self->context);
-
   z_trace(NULL, "Entering poll;");
   rc = pollfunc(self->pollfd, fdnum, polltimeout);
   z_trace(NULL, "Returning from poll;");
 
-  g_main_context_check(self->context, max_priority, self->pollfd, fdnum);
-  g_main_context_dispatch(self->context);
+  if (g_main_context_check(self->context, max_priority, self->pollfd, fdnum))
+    g_main_context_dispatch(self->context);
 
   if (rc == -1 && !z_errno_is(EINTR))
-    {
-      z_leave();
-      return 0;
-    }
+    z_return(0);
 
   if (rc == 0 && polltimeout == timeout)
     {
       z_errno_set(ETIMEDOUT);
-      z_leave();
-      return 0;
+      z_return(0);
     }
-
-  z_leave();
-  return 1;
+  z_return(1);
 }
 
 /**
- * z_poll_wakeup:
- * @s: ZPoll instance
- *
  * Wake up a running poll loop using its wakeup pipe.
+ *
+ * @param[in] s ZPoll instance
  **/
 void
 z_poll_wakeup(ZPoll *s)
@@ -411,20 +359,18 @@ z_poll_wakeup(ZPoll *s)
   ZPollSource *src;
   
   z_enter();
-  
   src = (ZPollSource *)self->wakeup;
   src->wakeup = TRUE;
   g_main_context_wakeup(self->context);
-  z_leave();
+  z_return();
 }
 
 /**
- * z_poll_is_running:
- * @s: ZPoll instance
- *
  * Checks whether z_poll_quit was called earlier on this ZPoll object.
  *
- * Returns: TRUE if the poll is still running
+ * @param[in] s ZPoll instance
+ *
+ * @returns TRUE if the poll is still running
  **/
 gboolean
 z_poll_is_running(ZPoll *s)
@@ -432,16 +378,15 @@ z_poll_is_running(ZPoll *s)
   ZRealPoll *self = (ZRealPoll *) s;
   
   z_enter();
-  z_leave();
-  return !self->quit;
+  z_return(!self->quit);
 }
 
 /**
- * z_poll_quit:
- * @s: ZPoll instance
+ * Indicate that this poll loop is to be ended.
  * 
- * Indicate that this poll loop is to be ended. Can be called from a
- * a thread different from the one running poll.
+ * @param[in] s ZPoll instance
+ *
+ * Can be called from thread different from the one running poll.
  **/
 void
 z_poll_quit(ZPoll *s)
@@ -449,18 +394,15 @@ z_poll_quit(ZPoll *s)
   ZRealPoll *self = (ZRealPoll *) s;  
   
   z_enter();
-  
   self->quit = TRUE;
   z_poll_wakeup(s);
-  
-  z_leave();
+  z_return();
 }
 
 /**
- * z_poll_get_context:
- * @s: ZPoll instance
- *
  * Return the underlying GMainContext.
+ *
+ * @param[in] s ZPoll instance
  **/
 GMainContext *
 z_poll_get_context(ZPoll *s)
@@ -468,6 +410,5 @@ z_poll_get_context(ZPoll *s)
   ZRealPoll *self = (ZRealPoll *) s;
   
   z_enter();
-  z_leave();
-  return self->context;
+  z_return(self->context);
 }
