@@ -39,6 +39,8 @@
 #include <pwd.h>
 #include <grp.h>
 
+#include <zorp/misc.h>
+
 #if HAVE_SYS_PRCTL_H
 #  include <sys/prctl.h>
 #endif
@@ -787,6 +789,7 @@ z_process_change_user(void)
 static gboolean
 z_process_change_caps(void)
 {
+#if ZORPLIB_ENABLE_CAPS
   if (process_opts.caps)
     {
       cap_t cap = cap_from_text(process_opts.caps);
@@ -806,6 +809,7 @@ z_process_change_caps(void)
         }
       zorp_caps = process_opts.caps;
     }
+#endif
   return TRUE;
 }
 
@@ -826,7 +830,7 @@ z_process_change_dir(void)
       else if (process_opts.pidfile_dir)
         cwd = process_opts.pidfile_dir;
       if (cwd)
-        chdir(cwd);
+        IGNORE_UNUSED_RESULT(chdir(cwd));
     }
     
   /* this check is here to avoid having to change directory early in the startup process */
@@ -834,7 +838,7 @@ z_process_change_dir(void)
     {
       gchar buf[256];
       
-      getcwd(buf, sizeof(buf));
+      IGNORE_UNUSED_RESULT(getcwd(buf, sizeof(buf)));
       z_process_message("Unable to write to current directory, core dumps will not be generated; dir='%s', error='%s'", buf, g_strerror(errno));
     }
   
@@ -874,7 +878,7 @@ z_process_send_result(guint ret_num)
   if (*fd != -1)
     {
       buf_len = g_snprintf(buf, sizeof(buf), "%d\n", ret_num);
-      write(*fd, buf, buf_len);
+      IGNORE_UNUSED_RESULT(write(*fd, buf, buf_len));
       close(*fd);
       *fd = -1;
     }  
