@@ -573,6 +573,8 @@ z_stream_buf_watch_dispatch(ZStream *s, GSource *src G_GNUC_UNUSED)
 static void
 z_stream_buf_set_child(ZStream *s, ZStream *new_child)
 {
+  z_stream_ref(s);
+
   Z_SUPER(s, ZStream)->set_child(s, new_child);
 
   if (new_child)
@@ -582,6 +584,8 @@ z_stream_buf_set_child(ZStream *s, ZStream *new_child)
       z_stream_set_callback(new_child, G_IO_PRI, z_stream_buf_pri_callback, z_stream_ref(s), (GDestroyNotify) z_stream_unref);
       z_stream_set_nonblock(new_child, TRUE);
     }
+
+  z_stream_unref(s);
 }
 
 /**
@@ -663,11 +667,4 @@ ZStreamFuncs z_stream_buf_funcs =
 /**
  * ZStreamBuf class descriptor.
  **/
-ZClass ZStreamBuf__class =
-{
-  Z_CLASS_HEADER,
-  &ZStream__class,
-  "ZStreamBuf",
-  sizeof(ZStreamBuf),
-  &z_stream_buf_funcs.super
-};
+Z_CLASS_DEF(ZStreamBuf, ZStream, z_stream_buf_funcs);
