@@ -687,7 +687,7 @@ z_stream_line_write_method(ZStream     *stream,
  **/
 static GIOStatus
 z_stream_line_write_pri_method(ZStream     *stream,
-                               const gchar *buf,
+                               const void *buf,
                                gsize       count,
                                gsize       *bytes_written,
                                GError      **error)
@@ -878,6 +878,19 @@ z_stream_line_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen
       ret = z_stream_ctrl_method(s, function, value, vlen);
       break;
 
+    case ZST_CTRL_GET_BUFFERED_BYTES:
+      if (vlen == sizeof(gsize))
+        {
+          gsize *size = (gsize *) value;
+
+          *size += self->end - self->pos;
+
+          ret = TRUE;
+          if (s->child)
+            ret = z_stream_ctrl(s->child, ZST_CTRL_MSG_FORWARD | function, value, vlen);
+        }
+      break;
+
     default:
       ret = z_stream_ctrl_method(s, ZST_CTRL_MSG_FORWARD | function, value, vlen);
       break;
@@ -1037,6 +1050,8 @@ z_stream_line_extra_restore_method(ZStream *s, gpointer extra)
 static void
 z_stream_line_set_child(ZStream *s, ZStream *new_child)
 {
+  z_stream_ref(s);
+
   Z_SUPER(s, ZStream)->set_child(s, new_child);
   
   if (new_child)
@@ -1045,6 +1060,8 @@ z_stream_line_set_child(ZStream *s, ZStream *new_child)
       z_stream_set_callback(new_child, G_IO_OUT, z_stream_line_write_callback, z_stream_ref(s), (GDestroyNotify) z_stream_unref);
       z_stream_set_callback(new_child, G_IO_PRI, z_stream_line_pri_callback, z_stream_ref(s), (GDestroyNotify) z_stream_unref);
     }
+
+  z_stream_unref(s);
 }
 
 /**
@@ -1120,11 +1137,4 @@ ZStreamFuncs z_stream_line_funcs =
 /**
  * ZStreamLine class descriptor.
  **/
-ZClass ZStreamLine__class = 
-{
-  Z_CLASS_HEADER,
-  &ZStream__class,
-  "ZStreamLine",
-  sizeof(ZStreamLine),
-  &z_stream_line_funcs.super,
-};
+Z_CLASS_DEF(ZStreamLine, ZStream, z_stream_line_funcs);
