@@ -15,6 +15,7 @@
  ***************************************************************************/
 
 #include <zorp/stream.h>
+#include <zorp/streamssl.h>
 #include <zorp/log.h>
 #include <zorp/ssl.h>
 #include <zorp/zorplib.h>
@@ -100,28 +101,12 @@ z_stream_ssl_pri_callback(ZStream *stream G_GNUC_UNUSED, GIOCondition poll_cond,
 /* virtual functions */
 
 static GIOStatus
-z_stream_ssl_read_method(ZStream *s, void *buf, gsize count,gsize *bytes_read, GError **error)
+z_stream_ssl_read_method_impl(ZStreamSsl *self, void *buf, gsize count, gsize *bytes_read, GError **error)
 {
-  ZStreamSsl *self = Z_CAST(s, ZStreamSsl);
   gint result;
   gint ssl_err;
 
   z_enter();
-  g_return_val_if_fail ((error == NULL) || (*error == NULL), G_IO_STATUS_ERROR);
-
-  if (self->what_if_called == CALL_WRITE_WHEN_READ)
-    {
-      /*LOG
-        This message indicates an internal error. Please report this event to the Balabit
-        QA Team (devel@balabit.com).
-       */
-      z_log(NULL, CORE_ERROR, 2, "Internal error; error='Read called, when only write might be called'");
-    }
-
-  if (self->shutdown)
-    z_return(G_IO_STATUS_EOF);
-
-  self->super.child->timeout = self->super.timeout;
   result = SSL_read(self->ssl->ssl, buf, count);
 
   if (result < 0)
@@ -153,7 +138,7 @@ z_stream_ssl_read_method(ZStream *s, void *buf, gsize count,gsize *bytes_read, G
           /*LOG
             This message indicates that an OS level error occurred during the SSL read. 
            */
-          g_set_error(error, G_IO_CHANNEL_ERROR, g_io_channel_error_from_errno(errno), g_strerror(errno));
+          g_set_error(error, G_IO_CHANNEL_ERROR, g_io_channel_error_from_errno(errno), "%s", g_strerror(errno));
           z_return(G_IO_STATUS_ERROR);
 
         case SSL_ERROR_SSL:
@@ -186,23 +171,44 @@ z_stream_ssl_read_method(ZStream *s, void *buf, gsize count,gsize *bytes_read, G
 }
 
 static GIOStatus
-z_stream_ssl_write_method(ZStream *s, const void *buf, gsize count, gsize *bytes_written, GError **error)
+z_stream_ssl_read_method(ZStream *s, void *buf, gsize count,gsize *bytes_read, GError **error)
 {
   ZStreamSsl *self = Z_CAST(s, ZStreamSsl);
   gint result;
+
+  z_enter();
+  g_return_val_if_fail ((error == NULL) || (*error == NULL), G_IO_STATUS_ERROR);
+
+  if (self->what_if_called == CALL_WRITE_WHEN_READ)
+    {
+      /*LOG
+        This message indicates an internal error. Please report this event to the Balabit
+        QA Team (devel@balabit.com).
+       */
+      z_log(NULL, CORE_ERROR, 2, "Internal error; error='Read called, when only write might be called'");
+    }
+
+  if (self->shutdown)
+    z_return(G_IO_STATUS_EOF);
+
+  self->super.child->timeout = self->super.timeout;
+
+  if (self->ssl)
+    result = z_stream_ssl_read_method_impl(self, buf, count, bytes_read, error);
+  else
+    result = z_stream_read(self->super.child, buf, count, bytes_read, error);
+
+  z_return(result);
+}
+
+
+static GIOStatus
+z_stream_ssl_write_method_impl(ZStreamSsl *self, const void *buf, gsize count, gsize *bytes_written, GError **error)
+{
+  gint result;
   gint ssl_err;
 
   z_enter();
-  g_return_val_if_fail ((error == NULL) || (*error == NULL), G_IO_STATUS_ERROR);
-  if (self->shutdown)
-    {
-      g_set_error(error, G_IO_CHANNEL_ERROR,
-                  g_io_channel_error_from_errno (ENOTCONN),
-                  g_strerror (ENOTCONN));
-      z_return(G_IO_STATUS_ERROR);
-    }
-
-  self->super.child->timeout = self->super.timeout;
   result = SSL_write(self->ssl->ssl, buf, count);
 
   if (result < 0)
@@ -229,7 +235,7 @@ z_stream_ssl_write_method(ZStream *s, const void *buf, gsize count, gsize *bytes
           /*LOG
             This message indicates that an OS level error occurred during the SSL write.
            */
-          g_set_error(error, G_IO_CHANNEL_ERROR, g_io_channel_error_from_errno(errno), g_strerror(errno));
+          g_set_error(error, G_IO_CHANNEL_ERROR, g_io_channel_error_from_errno(errno), "%s", g_strerror(errno));
           z_return(G_IO_STATUS_ERROR);
 
         case SSL_ERROR_SSL:
@@ -240,7 +246,7 @@ z_stream_ssl_write_method(ZStream *s, const void *buf, gsize count, gsize *bytes
           /*LOG
             This message indicates that an internal SSL error occurred during the SSL write.
            */
-          g_set_error(error, G_IO_CHANNEL_ERROR, G_IO_CHANNEL_ERROR_FAILED, self->error);
+          g_set_error(error, G_IO_CHANNEL_ERROR, G_IO_CHANNEL_ERROR_FAILED, "%s", self->error);
           z_return(G_IO_STATUS_ERROR);
         }
     }
@@ -255,6 +261,33 @@ z_stream_ssl_write_method(ZStream *s, const void *buf, gsize count, gsize *bytes
   z_return(G_IO_STATUS_NORMAL);
 }
 
+static GIOStatus
+z_stream_ssl_write_method(ZStream *s, const void *buf, gsize count, gsize *bytes_written, GError **error)
+{
+  ZStreamSsl *self = Z_CAST(s, ZStreamSsl);
+  gint result;
+
+  z_enter();
+  g_return_val_if_fail ((error == NULL) || (*error == NULL), G_IO_STATUS_ERROR);
+  if (self->shutdown)
+    {
+      g_set_error(error, G_IO_CHANNEL_ERROR,
+                  g_io_channel_error_from_errno (ENOTCONN),
+                  "%s",
+                  g_strerror (ENOTCONN));
+      z_return(G_IO_STATUS_ERROR);
+    }
+
+  self->super.child->timeout = self->super.timeout;
+
+  if (self->ssl)
+    result = z_stream_ssl_write_method_impl(self, buf, count, bytes_written, error);
+  else
+    result = z_stream_write(self->super.child, buf, count, bytes_written, error);
+
+  z_return(result);
+}
+
 /**
  * Close SSL connection.
  *
@@ -296,7 +329,7 @@ z_stream_ssl_shutdown_method(ZStream *s, int i, GError **error)
       
       
       z_stream_set_nonblock(s, FALSE);
-      if (SSL_shutdown(self->ssl->ssl) == 0)
+      if (self->ssl && SSL_shutdown(self->ssl->ssl) == 0)
         {
           /* if SSL_shutdown returns 0 it means that we still need to
            * process the shutdown alert by the peer, and to do that we need
@@ -307,7 +340,8 @@ z_stream_ssl_shutdown_method(ZStream *s, int i, GError **error)
       z_stream_set_nonblock(s, nonblock);
       z_stream_set_timeout(s, original_timeout);
 
-      ERR_clear_error();
+      if (self->ssl)
+        ERR_clear_error();
       self->shutdown = TRUE;
     }
   res = z_stream_shutdown(self->super.child, i, error);
@@ -328,7 +362,7 @@ static gboolean
 z_stream_ssl_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
 {
   ZStreamSsl *self G_GNUC_UNUSED = Z_CAST(s, ZStreamSsl);
-  gboolean ret;
+  gboolean ret = FALSE;
   
   z_enter();
   switch (ZST_CTRL_MSG(function))
@@ -339,6 +373,31 @@ z_stream_ssl_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
       ret = z_stream_ctrl_method(s, function, value, vlen);
       break;
 
+    case ZST_CTRL_SSL_SET_SESSION:
+      if (vlen == sizeof(ZSSLSession *))
+        {
+          ZSSLSession *ssl = (ZSSLSession *) value;
+          BIO *bio;
+
+          self->ssl = z_ssl_session_ref(ssl);
+
+          if (self->super.child)
+            {
+              bio = z_ssl_bio_new(self->super.child);
+              SSL_set_bio(self->ssl->ssl, bio, bio);
+            }
+        }
+      break;
+
+    case ZST_CTRL_GET_BUFFERED_BYTES:
+      /* we stop propagation here: this ctrl message queries the bytes
+         buffered *above* the SSL layer. */
+      if (vlen == sizeof(gsize))
+        {
+          ret = TRUE;
+        }
+      break;
+
     default:
       ret = z_stream_ctrl_method(s, ZST_CTRL_MSG_FORWARD | function, value, vlen);
       break;
@@ -360,10 +419,25 @@ z_stream_ssl_watch_prepare(ZStream *s, GSource *src G_GNUC_UNUSED, gint *timeout
           *timeout = 0;
           z_return(TRUE);
         }
-      if (SSL_pending(self->ssl->ssl))
+      if (self->ssl)
         {
-          *timeout = 0;
-	  z_return(TRUE);
+          if (SSL_pending(self->ssl->ssl))
+            {
+              *timeout = 0;
+              z_return(TRUE);
+            }
+        }
+      else
+        {
+          z_stream_set_cond(s->child, G_IO_IN, s->want_read);
+          z_stream_set_cond(s->child, G_IO_PRI, s->want_pri);
+          if (s->want_write)
+            {
+              z_stream_set_cond(s->child, G_IO_OUT, TRUE);
+              z_return(TRUE);
+            }
+          else
+            z_stream_set_cond(s->child, G_IO_OUT, FALSE);
         }
     }
   z_return(FALSE);
@@ -378,8 +452,11 @@ z_stream_ssl_watch_check(ZStream *s, GSource *src G_GNUC_UNUSED)
   z_enter();
   if (s->want_read)
     {
-      if (SSL_pending(self->ssl->ssl))
-        z_return(TRUE);
+      if (self->ssl)
+        {
+          if (SSL_pending(self->ssl->ssl))
+            z_return(TRUE);
+        }
     }
   z_return(FALSE);
 }
@@ -401,18 +478,25 @@ z_stream_ssl_set_child(ZStream *s, ZStream *new_child)
 {
   ZStreamSsl *self = Z_CAST(s, ZStreamSsl);
   BIO *bio;
+
+  z_stream_ref(s);
   
   Z_SUPER(s, ZStream)->set_child(s, new_child);
 
   if (self->super.child)
     {
-      bio = z_ssl_bio_new(self->super.child);
-      SSL_set_bio(self->ssl->ssl, bio, bio);
+      if (self->ssl)
+        {
+          bio = z_ssl_bio_new(self->super.child);
+          SSL_set_bio(self->ssl->ssl, bio, bio);
+        }
 
       z_stream_set_callback(self->super.child, G_IO_IN, z_stream_ssl_read_callback, z_stream_ref(s), (GDestroyNotify) z_stream_unref);
       z_stream_set_callback(self->super.child, G_IO_OUT, z_stream_ssl_write_callback, z_stream_ref(s), (GDestroyNotify) z_stream_unref);
       z_stream_set_callback(self->super.child, G_IO_PRI, z_stream_ssl_pri_callback, z_stream_ref(s), (GDestroyNotify) z_stream_unref);
     }
+
+  z_stream_unref(s);
 }
 
 ZStream *
@@ -422,7 +506,9 @@ z_stream_ssl_new(ZStream *child, ZSSLSession *ssl)
 
   z_enter();
   self = Z_CAST(z_stream_new(Z_CLASS(ZStreamSsl), "", G_IO_IN|G_IO_OUT), ZStreamSsl);
-  self->ssl = z_ssl_session_ref(ssl);
+
+  if (ssl)
+    self->ssl = z_ssl_session_ref(ssl);
   z_stream_set_child(&self->super, child);
   z_return((ZStream *) self);
 }
@@ -434,7 +520,8 @@ z_stream_ssl_free_method(ZObject *s)
   ZStreamSsl *self = Z_CAST(s, ZStreamSsl);
 
   z_enter();
-  z_ssl_session_unref(self->ssl);
+  if (self->ssl)
+    z_ssl_session_unref(self->ssl);
   ERR_clear_error();
   z_stream_free_method(s);
   z_return();
@@ -453,7 +540,7 @@ ZStreamFuncs z_stream_ssl_funcs =
   z_stream_ssl_read_method,
   z_stream_ssl_write_method,
   NULL,
-  NULL,
+  z_stream_ssl_write_method,
   z_stream_ssl_shutdown_method,
   NULL, /* close */
   z_stream_ssl_ctrl_method,
@@ -473,11 +560,4 @@ ZStreamFuncs z_stream_ssl_funcs =
 /**
  * ZStreamSsl class descriptor
  **/
-ZClass ZStreamSsl__class =
-{
-  Z_CLASS_HEADER,
-  &ZStream__class,
-  "ZStreamSsl",
-  sizeof(ZStreamSsl),
-  &z_stream_ssl_funcs.super,
-};
+Z_CLASS_DEF(ZStreamSsl, ZStream, z_stream_ssl_funcs);
