@@ -28,31 +28,15 @@ gboolean z_fd_set_oobinline(int fd, gboolean enable);
 
 #if ZORPLIB_ENABLE_TOS
 
-gboolean z_fd_get_peer_tos(gint fd, gint *tos);
-gboolean z_fd_get_our_tos(gint fd, gint *tos);
-gboolean z_fd_set_our_tos(gint fd, gint tos);
+void z_fd_get_peer_tos(gint fd, guint8 *tos);
+void z_fd_get_our_tos(gint fd, guint8 *tos);
+void z_fd_set_our_tos(gint fd, guint8 tos);
 
 #else
 
-static inline gboolean
-z_fd_get_peer_tos(gint fd G_GNUC_UNUSED, gint *tos)
-{
-  *tos = -1;
-  return TRUE;
-}
-
-static inline gboolean
-z_fd_get_our_tos(gint fd G_GNUC_UNUSED, gint *tos)
-{
-  *tos = -1;
-  return TRUE;
-}
-
-static inline G_GNUC_CONST gboolean
-z_fd_set_our_tos(gint fd G_GNUC_UNUSED, gint tos G_GNUC_UNUSED)
-{
-  return TRUE;
-}
+#define z_fd_get_peer_tos(fd, tos)
+#define z_fd_get_our_tos(fd, tos)
+#define z_fd_set_our_tos(fd, tos)
 
 #endif
 
