@@ -40,6 +40,7 @@ extern "C" {
 # endif
 #endif
 
+#define IGNORE_UNUSED_RESULT(x) if ((x)) { }
 /**
  * Stores a character set in the form of a bitfield.
  *
@@ -97,7 +98,7 @@ z_format_data_dump(const gchar *session_id, const gchar *class_, gint level, con
 void 
 z_format_text_dump(const gchar *session_id, const gchar *class_, gint level, const void *buf, guint len);
 
-GString *g_string_assign_len(GString *s, gchar *val, gint len);
+GString *g_string_assign_len(GString *s, const gchar *val, gint len);
 
 gint g_time_val_compare(const GTimeVal *t1, const GTimeVal *t2);
 glong g_time_val_diff(const GTimeVal *t1, const GTimeVal *t2);
