@@ -18,8 +18,16 @@
 extern "C" {
 #endif
 
+#define ZST_CTRL_SSL_SET_SESSION     (0x01) | ZST_CTRL_SSL_OFS
+
 ZStream * z_stream_ssl_new(ZStream *stream, ZSSLSession *ssl);
 
+static inline void
+z_stream_ssl_set_session(ZStream *self, ZSSLSession *ssl)
+{
+  z_stream_ctrl(self, ZST_CTRL_SSL_SET_SESSION, ssl, sizeof(&ssl));
+}
+
 #ifdef __cplusplus
 }
 #endif
