@@ -20,12 +20,10 @@ extern "C" {
 
 #define MAX_THREAD_NAME 128
 
-/*+
-
-  ZThread encapsulates a simple thread, and adds some meta information
-  like thread name, thread id etc.
-
-  +*/
+/**
+ * ZThread encapsulates a simple thread, and adds some meta information
+ * like thread name, thread id etc.
+ **/
 typedef struct _ZThread
 {
   GThread *thread;
@@ -35,15 +33,15 @@ typedef struct _ZThread
   gpointer arg;
 } ZThread;
 
-/* extern GList *threads; */
-extern gint max_threads, idle_threads;
-extern gboolean use_threadpools;
-
 void z_thread_register_start_callback(GFunc func, gpointer user_data);
 void z_thread_register_stop_callback(GFunc func, gpointer user_data);
 
 gboolean z_thread_new(gchar *name, GThreadFunc func, gpointer arg);
 
+void z_thread_enable_threadpools(gint idle);
+void z_thread_set_max_threads(gint max);
+void z_thread_set_max_stack_size(gint stack_size);
+
 void z_thread_init(void);
 void z_thread_destroy(void);
 ZThread *z_thread_self(void);
