@@ -12,17 +12,14 @@
 #ifndef ZORPLIB_H_INCLUDED
 #define ZORPLIB_H_INCLUDED
 
-
-#include <glib.h>
-
-#ifdef G_OS_WIN32
+#if 0
 #  ifdef COMPILING_LIBZORPLL
-#    define LIBZORPLL_DLLIMPEX __declspec(dllexport)
+#    define LIBZORPLL_EXTERN __declspec(dllexport)
 #  else
-#    define LIBZORPLL_DLLIMPEX __declspec(dllimport)
+#    define LIBZORPLL_EXTERN __declspec(dllimport)
 #  endif
 #else
-#  define LIBZORPLL_DLLIMPEX
+#  define LIBZORPLL_EXTERN extern 
 #endif
 
 #include <zorp/zorplibconfig.h>
@@ -30,6 +27,8 @@
 
 #include <zorp/memtrace.h>
 
+#include <glib.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
